<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00012000MedicaoFaturamentoFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
     /**
     * Retorna os medidas variáveis dos itens da medição.
     */
     
    public function getMedidasVariaveisMedicao(array $params) {
        $sql = "SELECT DISTINCT
                       MED.IDELEMENTORAIZ,
                       ELE.NMSERVORCA AS NMELEMENTORAIZ,
                       MED.CDTPMEDIDAVARIAVEL,
                       MEDIDA.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                       MED.QTMEDIDAVARIAVEL,
                       MED.IDMULTIPVARIACAO,
                       MED.QTFRANQUIA,
                       MED.CDSERVORCA
                  FROM FCT_MEDVARMEDICAOFATURA MED
                 INNER JOIN FCT_SERVICOORCA ELE
                    ON MED.IDELEMENTORAIZ = ELE.IDELEMENTO
                   AND MED.NRORG          = ELE.NRORG
                  LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                    ON MED.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                   AND MED.NRORG              = MEDIDA.NRORG
                 WHERE MED.NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND MED.NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
      /**
     * Retorna os Itens da medição.
     */
    
    public function getItensMedicao(array $params) {
        $sql = "SELECT ITMED.NRITMEDICAOFATURA,
                       ITMED.NRMEDICAOFATURA,
                       ITMED.NRSEQCONTRATO,
                       ITMED.NRITEMOCONTRATO,
                       ITMED.CDSERVORCA,
                       ITMED.CDPRODFATURA,
                       ITMED.IDSITUACAO,
                       ITMED.VRTOTITEMCONT,
                       ITMED.VRITEMCONT,
                       NVL((SELECT DECODE(SUM(VRDESCITEM), 0, NULL, SUM(VRDESCITEM))
                                 FROM FCT_ITNEGOCIACAOCONTRATO
                                 WHERE NRITEMCONTRATOPAI     = ITNEG.NRITEMOCONTRATO
                                    AND NRNEGOCIACAOCONTRATO = ITNEG.NRNEGOCIACAOCONTRATO), 0) AS VRDESCITEM,
                       DECODE(NEG.VRTOTITEM, 0, 0, (NVL(NEG.VRDESCITEM, 0) / NEG.VRTOTITEM * 100)) AS VRPERCDESCNEGO,
                       NEG.VRDESCITEM AS VRDESCITEMNEGO,
                       NEG.VRTOTITEMCOMDESC AS VRTOTITEMCOMDESCNEGO,
                       NEG.IDTPDESCONTO AS IDTPDESCONTONEG,
                       NEG.VRTOTITEM,
                       DES.VRPERCDESCITEM AS VRPERCDESCPERIODO,
                       DES.VRDESCITEM AS VRDESCITEMPORPERIODO,
                       DES.IDTPDESCONTO AS IDTPDESCPORPERIODO
                  FROM FCT_ITMEDICAOFATURA ITMED
                 INNER JOIN FCT_MEDICAOFATURA MED
                    ON ITMED.NRMEDICAOFATURA = MED.NRMEDICAOFATURA
                   AND ITMED.NRORG           = MED.NRORG
                 INNER JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
                    ON ITMED.NRITEMOCONTRATO = ITNEG.NRITEMOCONTRATO
                   AND ITMED.NRORG           = ITNEG.NRORG
                 INNER JOIN FCT_NEGOCIACAOCONTRATO NEG
                    ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
                   AND ITNEG.NRORG                = NEG.NRORG
                  LEFT JOIN FCT_DESCPERIODONEGOCONT DES
                    ON NEG.NRNEGOCIACAOCONTRATO = DES.NRNEGOCIACAOCONTRATO
                   AND NEG.NRORG                = DES.NRORG
                   AND TRUNC(MED.DTINICOMPET)   < TRUNC(DES.DTFINALVALIDADEDESC)
                   AND DES.NRORDEMAPLICADESC    = (SELECT MIN(NRORDEMAPLICADESC)
                                                     FROM FCT_DESCPERIODONEGOCONT
                                                    WHERE NRNEGOCIACAOCONTRATO   = NEG.NRNEGOCIACAOCONTRATO
                                                      AND NRORG                  = ITMED.NRORG
                                                      AND TRUNC(MED.DTINICOMPET) < TRUNC(DTFINALVALIDADEDESC))
                 WHERE ITMED.NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND ITMED.NRORG           = :NRORG
                   AND ITNEG.NRITEMCONTRATOPAI IS NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Atualiza os valores dos itens da medição.
     */
    public function updateItensMedicao(array $params) {
        $sql = "UPDATE FCT_ITMEDICAOFATURA
                   SET VRTOTITEMCONT      = ROUND(:VRTOTITEMCONT, 2),
                       IDSITUACAO         = :IDSITUACAO,
                       VRITEMCONT         = ROUND(:VRITEMCONT, 2),
                       VRNEGCONTR         = :VRNEGCONTR,
                       NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR,
                       DTULTATU           = SYSDATE,
                       NRORGULTATU        = :NRORG,
                       CDOPERULTATU       = :CDOPERULTATU
                 WHERE NRITMEDICAOFATURA = :NRITMEDICAOFATURA
                   AND NRMEDICAOFATURA   = :NRMEDICAOFATURA
                   AND NRITEMOCONTRATO   = :NRITEMOCONTRATO
                   AND CDSERVORCA        = :CDSERVORCA
                   AND NRORG             = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        
    }
    
    /**
     * Retorna os Itens da Contrato.
     */
    public function getItensContrato(array $params) {
        $sql = "SELECT DISTINCT ITEM.CDTPMEDIDASERV,
                       ITEM.CDTPMEDIDAVARIAVEL,
                       ITEM.QTMEDIDACONTR,
                       ITEM.NRSEQCONTRATO,
                       ITEM.NRITEMOCONTRATO,
                       ITEM.CDSERVORCA,
                       SERV.NMSERVORCA,
                       ITEM.VRUNITITEMCONTTBLPRECO AS VRUNITITEMORCATBLPRECO,
                       ITEM.VRUNITITEMCONT AS VRUNITITEMORCA,
                       ITEM.IDELEMENTO AS IDELEMENTORAIZ,
                       ITEM.NMITEMCONTRATO AS NMELEMENTO,
                       DECODE(VARIACAO.IDMULTIPVARIACAO, 'N', ITEM.VRUNITITEMCONT, ITEM.VRUNITITEMCONT * QTMEDIDACONTR) AS VRTOTAL,
                       ITEM.IDTPDESCONTO AS IDTPDESCORCA,
                       MEDVARMEDICAO.QTMEDIDAVARIAVEL,
                       MEDVARMEDICAO.QTMEDIDAMEDICAO,
                       MEDVARMEDICAO.QTFRANQUIA,
                       MEDVARMEDICAO.QTMEDIDAVENDIDA,
                       VARIACAO.VRPRECOVARIACAO,
                       VARIACAOVEND.VRPRECOVARIACAO AS VRPRECOVARIACAOVEND,
                       DECODE(NVL(VARIACAO.IDMULTIPVARIACAO, 'N'), 'S', 'Sim', 'Não') AS IDMULTIPVARIACAO,
                       ITNEG.IDTPDESCONTO,
                       ITNEG.VRDESCITEM,
                       ITNEG.VRPERCDESCITEM,
                       ITNEG.VRTOTITEM,
                       ITEM.VRTOTITEMCONT
                  FROM FCT_ITEMCONTRATO ITEM
                  JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
                    ON ITEM.NRITEMOCONTRATO = ITNEG.NRITEMOCONTRATO
                   AND ITEM.NRORG          = ITNEG.NRORG
                 INNER JOIN FCT_SERVICOORCA SERV
                    ON ITEM.CDSERVORCA = SERV.CDSERVORCA
                   AND ITEM.NRORG      = SERV.NRORG
                 INNER JOIN FCT_MEDVARMEDICAOFATURA MEDVARMEDICAO
                    ON (ITEM.IDELEMENTO = MEDVARMEDICAO.IDELEMENTORAIZ 
                    OR MEDVARMEDICAO.IDELEMENTORAIZ IS NULL AND ITEM.IDELEMENTO IS NULL)
                   AND ITEM.CDSERVORCA               = MEDVARMEDICAO.CDSERVORCA
                   AND ITEM.CDTPMEDIDAVARIAVEL       = MEDVARMEDICAO.CDTPMEDIDAVARIAVEL
                   AND MEDVARMEDICAO.NRMEDICAOFATURA = :P_NRMEDICAOFATURA
                  LEFT JOIN FCT_VARPRECBASESERVCONT VARIACAO
                    ON ITEM.NRSEQCONTRATO      = VARIACAO.NRSEQCONTRATO 
                   AND ITEM.NRORG              = VARIACAO.NRORG
                   AND ITEM.CDSERVORCA         = VARIACAO.CDSERVORCA
                   AND ITEM.CDTPMEDIDAVARIAVEL = VARIACAO.CDTPMEDIDAVARIAVEL
                   AND (ITEM.CDTPMEDIDASERV    = VARIACAO.CDTPMEDIDASERV OR MEDVARMEDICAO.IDELEMENTORAIZ IS NULL)
                   AND (DECODE(MEDVARMEDICAO.QTMEDIDAMEDICAO + MEDVARMEDICAO.QTFRANQUIA, 0, 1, MEDVARMEDICAO.QTMEDIDAMEDICAO + MEDVARMEDICAO.QTFRANQUIA) BETWEEN VARIACAO.QTMEDIDAINIVARIACAO AND VARIACAO.QTMEDIDAFIMVARIACAO)
                  LEFT JOIN FCT_VARPRECBASESERVCONT VARIACAOVEND
                    ON ITEM.NRSEQCONTRATO      = VARIACAOVEND.NRSEQCONTRATO 
                   AND ITEM.NRORG              = VARIACAOVEND.NRORG
                   AND ITEM.CDSERVORCA         = VARIACAOVEND.CDSERVORCA
                   AND ITEM.CDTPMEDIDAVARIAVEL = VARIACAOVEND.CDTPMEDIDAVARIAVEL
                   AND (ITEM.CDTPMEDIDASERV     = VARIACAOVEND.CDTPMEDIDASERV OR MEDVARMEDICAO.IDELEMENTORAIZ IS NULL)
                   AND (DECODE(NVL(MEDVARMEDICAO.QTMEDIDAVENDIDA, 0), 0, 1, MEDVARMEDICAO.QTMEDIDAVENDIDA) BETWEEN VARIACAOVEND.QTMEDIDAINIVARIACAO AND VARIACAOVEND.QTMEDIDAFIMVARIACAO)
                 WHERE (MEDVARMEDICAO.IDELEMENTORAIZ IS NOT NULL OR ITEM.IDELEMENTO IS NOT NULL 
                    OR MEDVARMEDICAO.IDELEMENTORAIZ IS NULL AND ITEM.IDELEMENTO IS NULL)
                   AND ITEM.NRSEQCONTRATO = :P_NRSEQCONTRATO
                   AND ITEM.NRORG         = :P_NRORG
                   AND ITEM.CDSERVORCA    = :P_CDSERVORCA";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna a situação das variações de Medições.
     */
    public function getSituacaoMedicaoVar(array $params) {
        $sql =  "SELECT COUNT(*) AS PENDENTE
                 FROM FCT_MEDVARMEDICAOFATURA
                 WHERE IDCALCULADO = 'N'
                   AND NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND CDSERVORCA = :CDSERVORCA
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
     /**
     * Retorna a situação dos itens das Medições.
     */
    public function getSituacaoItMedicao(array $params) {
        $sql =  "SELECT SUM(PENDENTE) AS PENDENTE, SUM(PARCIAL) AS PARCIAL, SUM(CALCULADO) AS CALCULADO
                    FROM(
                    SELECT 0 AS PENDENTE , 0 AS PARCIAL, COUNT(*) AS CALCULADO
                     FROM FCT_ITMEDICAOFATURA
                     WHERE IDSITUACAO = 3
                       AND NRMEDICAOFATURA = :NRMEDICAOFATURA
                       AND NRORG = :NRORG
                    UNION ALL 
                    SELECT 0 AS PENDENTE, COUNT(*) AS PARCIAL,  0 AS CALCULADO
                     FROM FCT_ITMEDICAOFATURA
                     WHERE IDSITUACAO = 2
                       AND NRMEDICAOFATURA = :NRMEDICAOFATURA
                       AND NRORG = :NRORG
                    UNION ALL 
                    SELECT COUNT(*) AS PENDENTE , 0 AS PARCIAL,  0 AS CALCULADO
                     FROM FCT_ITMEDICAOFATURA
                     WHERE IDSITUACAO = 1
                       AND NRMEDICAOFATURA = :NRMEDICAOFATURA
                       AND NRORG = :NRORG
                    )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
     public function getSomaItMedicao(array $params) {
        $sql =  "SELECT SUM(VRITEMCONT) AS VRITEMCONT,
                        SUM(VRNEGCONTR) AS VRNEGCONTR,
                        SUM(NVL(VRNEGCONTR, VRITEMCONT)) AS VRCONSIDERADO
                   FROM FCT_ITMEDICAOFATURA
                  WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                    AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizaMedicao(array $params) {
        $sql = "UPDATE FCT_MEDICAOFATURA
                   SET VRMEDICAO       = ROUND(:VRMEDICAO, 2),
                       VRNEGCONTR      = ROUND(:VRNEGCONTR, 2),
                       IDSITUACAO      = :IDSITUACAO,
                       DSJUSTIFICACANC = NULL
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        
    }
    
    /**
     * Verifica se a medição é de um contrato agrupado
    **/
    public function getAgrupamentoCont(array $params){
        $sql = "SELECT COUNT(*) AS VALIDA
                FROM FCT_RELACAGRUPCONTFAT
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND NRORG           = :NRORG
                    AND NRMEDICAOFATURA = :NRMEDICAOFATURA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataSet = $std->fetchAll();
        return !empty($dataSet) ? $dataSet[0]["VALIDA"] : null;
    }
    
    /**
     * A função "validaFatura" valida se ja foi gerada parcela.
     */
    public function validaFatura(array $params) {
        $sql =  "SELECT COUNT(*) VALIDA
                   FROM FCT_FATURACONT
                  WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                    AND NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Atualiza medidas Variáveis*/
    public function atualizarMedVariavelOM(array $params) {
        $sql = "UPDATE FCT_MEDVARMEDICAOFATURA
                   SET QTMEDIDAVARIAVEL = ROUND(:QTMEDIDAVARIAVEL, 2),
                       QTMEDIDAMEDICAO  = ROUND(:QTFATURAMENTO - QTFRANQUIA, 2),
                       IDCALCULADO      = 'S',
                       DTCOLETAVALIDAOM = :DTCOLETA,
                       QTMEDIDAVALIDAOM = ROUND(:QTMEDIDAVARIAVEL, 2),
                       DTULTIMACOLETAOM = :DTCOLETA,
                       QTULTIMAMEDIDAOM = ROUND(:QTMEDIDAVARIAVEL, 2),
                       NRORGULTATU      = :NRORG,
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU 
                 WHERE NRMEDICAOFATURA           = :NRMEDICAOFATURA
                   AND CDTPMEDIDAVARIAVEL        = :CDTPMEDIDAVARIAVEL
                   AND NRORG                     = :NRORG
                   AND (SUBSTR(IDELEMENTORAIZ, 3) = :IDPRODUTOOM OR (IDELEMENTORAIZ IS NULL AND CDSERVORCA = :CDSERVORCA))";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
    }
    
    /* Atualiza data e quantidade da última de acordo com os dados enviados pelo OM */
    public function atualizaUltimaColetaOM(array $params) {
        $sql = "UPDATE FCT_MEDVARMEDICAOFATURA
                   SET DTULTIMACOLETAOM = :DTCOLETA,
                       QTULTIMAMEDIDAOM = ROUND(:QTMEDIDAVARIAVEL, 2),
                       NRORGULTATU      = :NRORG,
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU 
                 WHERE NRMEDICAOFATURA           = :NRMEDICAOFATURA
                   AND CDTPMEDIDAVARIAVEL        = :CDTPMEDIDAVARIAVEL
                   AND NRORG                     = :NRORG
                   AND (SUBSTR(IDELEMENTORAIZ, 3) = :IDPRODUTOOM OR (IDELEMENTORAIZ IS NULL AND CDSERVORCA = :CDSERVORCA))";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
    }
    
    /* Retorna a situação da medição de faturamento */
    public function ValidaSitMedicao(array $params) {
        $sql = "SELECT IDSITUACAO 
                  FROM FCT_MEDICAOFATURA
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND TRUNC(DTINICOMPET) = :DTINICOMPET
                   AND TRUNC(DTFINCOMPET) = :DTFINCOMPET
                   AND NRORG              = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }
    
    /* Retorna a quantidade da medida variável da medição de faturamento */
    public function ValidaQTtMedicaoConc(array $params) {
        $sql = "SELECT MAX(FCT_MEDVARMEDICAOFATURA.QTMEDIDAVARIAVEL) AS QTMEDIDAVARIAVEL, QTMEDIDAVENDIDA
                  FROM FCT_MEDVARMEDICAOFATURA
                  JOIN FCT_MEDICAOFATURA
                    ON FCT_MEDICAOFATURA.NRMEDICAOFATURA = FCT_MEDVARMEDICAOFATURA.NRMEDICAOFATURA
                   AND FCT_MEDICAOFATURA.NRORG           = FCT_MEDVARMEDICAOFATURA.NRORG 
                   AND FCT_MEDICAOFATURA.IDSITUACAO      = '3'
                 WHERE FCT_MEDVARMEDICAOFATURA.NRMEDICAOFATURA           = :NRMEDICAOFATURA
                   AND FCT_MEDVARMEDICAOFATURA.CDTPMEDIDAVARIAVEL        = :CDTPMEDIDAVARIAVEL
                   AND FCT_MEDVARMEDICAOFATURA.NRORG                     = :NRORG
                   AND (SUBSTR(FCT_MEDVARMEDICAOFATURA.IDELEMENTORAIZ, 3) = :IDPRODUTOOM OR (FCT_MEDVARMEDICAOFATURA.IDELEMENTORAIZ IS NULL AND FCT_MEDVARMEDICAOFATURA.CDSERVORCA = :CDSERVORCA))
                 GROUP BY QTMEDIDAVENDIDA";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }
    
    
    /* Retorna a quantidade da medida variável da medição de faturamento */
    public function ValidaQTtMedicao(array $params) {
        $sql = "SELECT MAX(FCT_MEDVARMEDICAOFATURA.QTMEDIDAVARIAVEL) AS QTMEDIDAVARIAVEL, QTMEDIDAVENDIDA
                  FROM FCT_MEDVARMEDICAOFATURA
                  JOIN FCT_MEDICAOFATURA
                    ON FCT_MEDICAOFATURA.NRMEDICAOFATURA = FCT_MEDVARMEDICAOFATURA.NRMEDICAOFATURA
                   AND FCT_MEDICAOFATURA.NRORG           = FCT_MEDVARMEDICAOFATURA.NRORG
                 WHERE FCT_MEDVARMEDICAOFATURA.NRMEDICAOFATURA           = :NRMEDICAOFATURA
                   AND FCT_MEDVARMEDICAOFATURA.CDTPMEDIDAVARIAVEL        = :CDTPMEDIDAVARIAVEL
                   AND FCT_MEDVARMEDICAOFATURA.NRORG                     = :NRORG
                   AND (SUBSTR(FCT_MEDVARMEDICAOFATURA.IDELEMENTORAIZ, 3) = :IDPRODUTOOM OR (FCT_MEDVARMEDICAOFATURA.IDELEMENTORAIZ IS NULL AND FCT_MEDVARMEDICAOFATURA.CDSERVORCA = :CDSERVORCA))
                 GROUP BY QTMEDIDAVENDIDA";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }
    
    /* Atualiza a conexão do banco de acordo com o connection enviado.
       Atualmente  utilizado para funcionamento da rotina "atualizarMedVariavelOM" do service "Sm00012000MedicaoFaturamentoService"*/
    public function atualizaConnection($connection){

        $this->connection = $connection;
    }
    
    
    /* Busca o nr da medição de faturamento de acordo com o contrato e o período.*/
    public function getNrMedicaoFatura(array $params) {
        
        $sql = "SELECT NRMEDICAOFATURA 
                  FROM FCT_MEDICAOFATURA
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND TRUNC(DTINICOMPET) = :DTINICOMPET
                   AND TRUNC(DTFINCOMPET) = :DTFINCOMPET
                   AND NRORG = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }
    
    public function getItensProposta(array $params) {
        $sql = "SELECT DISTINCT ITEM.CDTPMEDIDASERV,
                        ITEM.CDTPMEDIDAVARIAVEL,
                        ITEM.NRSEQORCAMENTO, 
                        ITEM.CDSERVORCA,
                        SERV.NMSERVORCA,
                        ITEM.VRUNITITEMORCATBLPRECO,
                        ITEM.VRUNITITEMORCA,
                        ITEM.IDELEMENTO AS IDELEMENTORAIZ,
                        ITEM.NMITEMORCAMENTO AS NMELEMENTO,
                        DECODE(VARIACAO.IDMULTIPVARIACAO, 'N', ITEM.VRUNITITEMORCA, ITEM.VRUNITITEMORCA * QTMEDIDAORCA) AS VRTOTAL,
                        ITEM.IDTPDESCORCA,
                        ITEM.VRPERCDESCORCA,
                        ITEM.VRDESCORCA,
                        FCT_MEDVARMEDICAOFATURA.QTMEDIDAVARIAVEL,
                        FCT_MEDVARMEDICAOFATURA.QTMEDIDAMEDICAO,
                        FCT_MEDVARMEDICAOFATURA.QTFRANQUIA,
                        VARIACAO.VRPRECOVARIACAO,
                        ITEM.VRTOTALITEMORCA AS VRTOTALVARIACAO,
                        DECODE(NVL(VARIACAO.IDMULTIPVARIACAO, 'N'), 'S', 'Sim', 'Não') AS IDMULTIPVARIACAO,
                        DECODE(NVL(VARIACAO.IDMULTIPVARIACAO, 'N'), 'N', ITEM.QTMEDIDAORCA, ITEM.VRUNITITEMORCATBLPRECO / VARIACAO.VRPRECOVARIACAO) AS QTMEDIDA
                FROM FCT_ITEMORCAMENTO ITEM
                INNER JOIN FCT_SERVICOORCA SERV
                ON ITEM.CDSERVORCA = SERV.CDSERVORCA
                AND ITEM.NRORG     = SERV.NRORG
                LEFT JOIN FCT_TIPOORCAMENTO TIPO
                ON ITEM.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                AND ITEM.NRORG          = TIPO.NRORG
                INNER JOIN FCT_MEDVARMEDICAOFATURA
                ON ITEM.IDELEMENTO                          = FCT_MEDVARMEDICAOFATURA.IDELEMENTORAIZ
                AND ITEM.CDSERVORCA                         = FCT_MEDVARMEDICAOFATURA.CDSERVORCA
                AND FCT_MEDVARMEDICAOFATURA.NRMEDICAOFATURA = :P_NRMEDICAOFATURA
                LEFT JOIN FCT_VARPRECBASESERVORCA VARIACAO
                ON ITEM.NRSEQORCAMENTO      = VARIACAO.NRSEQORCAMENTO
                AND ITEM.NRORG              = VARIACAO.NRORG
                AND ITEM.CDSERVORCA         = VARIACAO.CDSERVORCA
                AND ITEM.CDTPMEDIDASERV     = VARIACAO.CDTPMEDIDASERV
                AND ITEM.CDTPMEDIDAVARIAVEL = VARIACAO.CDTPMEDIDAVARIAVEL
                AND (FCT_MEDVARMEDICAOFATURA.QTMEDIDAVARIAVEL BETWEEN VARIACAO.QTMEDIDAINIVARIACAO AND VARIACAO.QTMEDIDAFIMVARIACAO)
                WHERE ITEM.IDELEMENTO IS NOT NULL
                AND ITEM.NRSEQORCAMENTO  = :P_NRSEQORCAMENTO
                AND ITEM.NRORG           = :P_NRORG
                AND ITEM.NRTIPOORCAMENTO = :P_NRTIPOORCAMENTO 
                AND ITEM.CDSERVORCA      = :P_CDSERVORCA";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getQuantidadeElementosNaoMedidosMedicao(array $params) {
        $sql = "SELECT COUNT(NRMEDVARMEDICAOFATURA) AS QTD
                  FROM FCT_MEDVARMEDICAOFATURA
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG           = :NRORG
                   AND IDCALCULADO     = 'N'
                   AND (NVL(QTMEDIDAVARIAVEL, 0) = 0 AND QTMEDIDAVALIDAOM IS NULL)";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function atualizarMedicaoSituacaoCancelada(array $params) {
        $sql = "UPDATE FCT_MEDICAOFATURA
                   SET IDSITUACAO      = '6',
                       DSJUSTIFICACANC = :DSJUSTIFICACANC,
                       DTULTATU        = SYSDATE,
                       NRORGULTATU     = :NRORGULTATU,
                       CDOPERULTATU    = :CDOPERULTATU
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG           = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function validaProporcional(array $params) {
        $sql = "SELECT TO_NUMBER(TO_CHAR(LAST_DAY(DTINICOMPET), 'DD')) AS PRAZOINT, 
                       (DTFINCOMPET - DTINICOMPET)+1 AS PRAZO
                  FROM FCT_MEDICAOFATURA
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    /* Verifica parametrização de periodo incompleto do faturamento */
    public function recebeParamFatIncompleto(array $params) {
        $sql = "SELECT IDTPCALCPARCINC
                  FROM FCT_TIPOORCAMENTO
                  JOIN FCT_CAPAPARAMFATESC
                    ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_CAPAPARAMFATESC.NRTIPOORCAMENTO
                   AND FCT_TIPOORCAMENTO.NRORG = FCT_CAPAPARAMFATESC.NRORG
                  WHERE FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = (SELECT NVL(FCT_CONTRATO.NRTIPOORCAMENTO, FCT_REFEORCACONT.NRTIPOORCAMENTO)
                                                               FROM FCT_CONTRATO
                                                               JOIN FCT_REFEORCACONT
                                                                 ON FCT_REFEORCACONT.NRSEQCONTRATO = FCT_CONTRATO.NRSEQCONTRATO
                                                                AND FCT_REFEORCACONT.NRORG = FCT_CONTRATO.NRORG
                                                              WHERE FCT_CONTRATO.NRSEQCONTRATO = :NRSEQCONTRATO
                                                                AND FCT_CONTRATO.NRORG = :NRORG)
                  AND FCT_TIPOORCAMENTO.NRORG = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    /* Verifica se a data inicial da primeira parcela é a mesma da data do contrato, se não for, calcula o prazo anterior à parcela inicial */
    public function verificaPeriodoIncompProRata(array $params) {
        $sql = "SELECT MIN(FCT_MEDICAOFATURA.DTINICOMPET) AS DTCOMPETMIN, MAX(FCT_MEDICAOFATURA.DTFINCOMPET) AS DTCOMPETMAX, 
                       (MIN(FCT_MEDICAOFATURA.DTINICOMPET) - NVL(FCT_CONTRATO.DTINIVIGENCONT, FCT_CONTRATO.DTINICIOCONTRATO)) AS PRAZO_INICONT,
                       TO_NUMBER(TO_CHAR(LAST_DAY(NVL(FCT_CONTRATO.DTINIVIGENCONT, FCT_CONTRATO.DTINICIOCONTRATO)), 'DD')) AS PRAZOINT
                  FROM FCT_MEDICAOFATURA
                  JOIN FCT_CONTRATO 
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCT_MEDICAOFATURA.NRSEQCONTRATO
                   AND FCT_CONTRATO.NRORG = FCT_MEDICAOFATURA.NRORG
                 WHERE FCT_MEDICAOFATURA.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND FCT_MEDICAOFATURA.NRORG = :NRORG
                GROUP BY FCT_CONTRATO.DTINIVIGENCONT, FCT_CONTRATO.DTINICIOCONTRATO";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    } 
    
    /* recebe periodo da medição de faturamento */
    public function recebeperiodoMedicao(array $params) {
        $sql = "SELECT DTINICOMPET, DTFINCOMPET
                  FROM FCT_MEDICAOFATURA
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getDadosMedicao(array $params) {
        $sql = "SELECT NRSEQCONTRATO,
                       IDSITUACAO,
                       TO_CHAR(DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET,
                       TO_CHAR(DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET,
                       NRPARCFATURA,
                       CDFILIALFATURA,
                       CDTIPOOPERFATURA,
                       CDSERIENF,
                       CDEMPRESA,
                       CDINSCESTA,
                       CDINSCMUNI
                  FROM FCT_MEDICAOFATURA
                 WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND NRORG           = :NRORG";
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getNegociacaoContratoParaElemento(array $params) {
        $sql = "SELECT PARC.NRPARCELASNEGCONTR,
                       PARC.VRPARCELANEGOCIA,
                       PARC.IDMULTIPLICAQTMED,
                       NEG.IDTIPONEGOCIA
                  FROM FCT_PARCELASNEGCONTR PARC
                 INNER JOIN FCT_NEGCONTRATO NEG
                    ON PARC.NRNEGCONTRATO = NEG.NRNEGCONTRATO
                 WHERE ( :DTINICOMPET BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA)
                        OR :DTFINCOMPET BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA) )
                   AND NEG.NRPARCNEGOCIO  = :NRPARCNEGOCIO
                   AND NEG.NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NEG.NRITEMELEMENTO = :NRITEMELEMENTO
                   AND NEG.NRORG          = :NRORG
                   AND NEG.IDATIVO        = 'S'
                   AND PARC.IDATIVO       = 'S'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizarMedidaVariavel(array $params) {
        $sql = "UPDATE FCT_MEDVARMEDICAOFATURA
                   SET IDCALCULADO        = :IDCALCULADO,
                       VRITEMCONT         = :VRITEMCONT,
                       VRITEMCONTCOMDESC  = :VRITEMCONTCOMDESC,
                       VRNEGCONTR         = :VRNEGCONTR,
                       NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR,
                       DTULTATU           = SYSDATE,
                       NRORGULTATU        = :NRORGULTATU,
                       CDOPERULTATU       = :CDOPERULTATU
                 WHERE NRMEDICAOFATURA    = :NRMEDICAOFATURA
                   AND IDELEMENTORAIZ     = :IDELEMENTORAIZ
                   AND CDSERVORCA         = :CDSERVORCA
                   AND CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                   AND NRORG              = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getNegociacaoContratoParaServico(array $params) {
        $sql = "SELECT PARC.NRPARCELASNEGCONTR,
                       PARC.VRPARCELANEGOCIA,
                       'N' AS IDMULTIPLICAQTMED,
                       NEG.IDTIPONEGOCIA
                  FROM FCT_PARCELASNEGCONTR PARC
                 INNER JOIN FCT_NEGCONTRATO NEG
                    ON PARC.NRNEGCONTRATO = NEG.NRNEGCONTRATO
                 WHERE ( :DTINICOMPET BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA)
                        OR :DTFINCOMPET BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA) )
                   AND NEG.NRPARCNEGOCIO   = :NRPARCNEGOCIO
                   AND NEG.NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND NEG.NRITEMOCONTRATO = :NRITEMOCONTRATO
                   AND NEG.NRORG           = :NRORG
                   AND NEG.IDATIVO         = 'S'
                   AND PARC.IDATIVO        = 'S'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getDadosNegociacaoMedicao(array $params) {
        $sql = "SELECT DISTINCT NEG.NRNEGOCIACAOCONTRATO,
                       NEG.VRTOTITEM,
                       NEG.IDTPDESCONTO,
                       NEG.VRDESCITEM,
                       NEG.VRPERCDESCITEM,
                       NEG.VRTOTITEMCOMDESC,
                       DES.VRPERCDESCITEM AS VRPERCDESCPERIODO,
                       DES.VRDESCITEM AS VRDESCITEMPORPERIODO,
                       DES.IDTPDESCONTO AS IDTPDESCPORPERIODO
                  FROM FCT_ITNEGOCIACAOCONTRATO ITNEG
                 INNER JOIN FCT_NEGOCIACAOCONTRATO NEG
                    ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
                   AND ITNEG.NRORG                = NEG.NRORG
                  LEFT JOIN FCT_DESCPERIODONEGOCONT DES
                    ON NEG.NRNEGOCIACAOCONTRATO = DES.NRNEGOCIACAOCONTRATO
                   AND NEG.NRORG                = DES.NRORG
                   AND :DTINICOMPET             < TRUNC(DES.DTFINALVALIDADEDESC)
                   AND DES.NRORDEMAPLICADESC = (SELECT MIN(NRORDEMAPLICADESC)
                                                  FROM FCT_DESCPERIODONEGOCONT
                                                 WHERE NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
                                                   AND NRORG                = NEG.NRORG
                                                   AND :DTINICOMPET         < TRUNC(DTFINALVALIDADEDESC))
                 WHERE ITNEG.NRITEMOCONTRATO IN (
                    SELECT NRITEMOCONTRATO
                      FROM FCT_ITMEDICAOFATURA
                     WHERE NRMEDICAOFATURA = :NRMEDICAOFATURA
                       AND NRORG           = :NRORG
                 )";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getItensMedicaoComSoma(array $params) {
        $sql = "SELECT IT.NRITMEDICAOFATURA,
                       IT.NRMEDICAOFATURA,
                       IT.NRSEQCONTRATO,
                       IT.NRITEMOCONTRATO,
                       IT.CDSERVORCA,
                       IT.CDPRODFATURA,
                       IT.IDSITUACAO,
                       IT.VRTOTITEMCONT,
                       IT.VRITEMCONT,
                       IT.VRNEGCONTR,
                       IT.NRPARCELASNEGCONTR,
                       MED.VRMEDICAO
                  FROM FCT_ITMEDICAOFATURA IT
                  JOIN FCT_MEDICAOFATURA MED
                    ON IT.NRMEDICAOFATURA = MED.NRMEDICAOFATURA
                   AND IT.NRORG           = MED.NRORG
                 WHERE IT.NRMEDICAOFATURA = :NRMEDICAOFATURA
                   AND IT.NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
}

?>