<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00004000PeriodoVigenciaContratoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Service\IntegracaoOmService;

class Sm00004000PeriodoVigenciaContratoService
{
    protected $connection;
    protected $PeriodoVigenciaContratoFactory;
    protected $Environment;
    protected $integracaoOmService;
    
    public function __construct (EntityManager $entityManager, Sm00004000PeriodoVigenciaContratoFactory $PeriodoVigenciaContratoFactory, Environment $Environment, IntegracaoOmService $integracaoOmService){
        
        $this->connection                      = $entityManager->getConnection();
        $this->PeriodoVigenciaContratoFactory  = $PeriodoVigenciaContratoFactory;
        $this->Environment                     = $Environment;
        $this->integracaoOmService             = $integracaoOmService;

    }
    
     /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
     
    public function alteraVigenciaContrato($dtInicVigContr, $dtFinVigContr,  $seqContrato){
        
        try{
            
            $paramsAlteraVigenciaContr = array(
                
                "DTINIVIGENCONT" =>  $dtInicVigContr,
                "DTFINVIGENCONT" =>  $dtFinVigContr,
                "NRSEQCONTRATO" => $seqContrato,
                "NRORG"  => $this->Environment->getNrOrg()
            );
            
            $this->PeriodoVigenciaContratoFactory->alteraVigenciaContrato($paramsAlteraVigenciaContr);
            
        
        }
        catch (\Exception $e) {
            throw $e;
        };
        
    }
    
    /**
     * A função "alteraVigenciaContrato" altera os dados de vigência do contrato
     */
     
    public function alteraVigenciaContratoNovo($nrorg, $idutilizacomp, $dtInicVigContr, $dtFinVigContr, $periodoextendido, $seqContrato){
        
        $this->connection->beginTransaction();
        
        try{
            $paramsAlteraVigenciaContr = array(
                
                "DTINIVIGENCONT" =>  $dtInicVigContr,
                "DTFINVIGENCONT" =>  $dtFinVigContr,
                "NRSEQCONTRATO" => $seqContrato,
                "NRORG"  => $this->Environment->getNrOrg()
            );
            
            $this->PeriodoVigenciaContratoFactory->alteraVigenciaContrato($paramsAlteraVigenciaContr);
            
            //Envia os Elementos para o OM
            $this->integracaoOmService->integracaoOM(null, $nrorg, $idutilizacomp, $periodoextendido, $seqContrato);
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        };
        
        
    }
    
     /**
     * A função "validaDadosProrrog" valida se ja existem alterações de vigência para o contrato
     */
     
    public function validaDadosProrrog($seqContrato){
    
        $paramsValidaDadosProrrog = array(
            
            "NRSEQCONTRATO" => $seqContrato,
            "NRORG"  => $this->Environment->getNrOrg()
        );
        
      $returnValidaDadosProrrog = $this->PeriodoVigenciaContratoFactory->validaDadosProrrog($paramsValidaDadosProrrog);
    
      return $returnValidaDadosProrrog[0]["NRSEQPRORRCONT"];    
        
    }
    
     /**
     * A função "alteraDadosProrrog" inativa as alterações de vigência do contrato
     */
     
    public function alteraDadosProrrog($seqContrato, $seqprorrog){
    
        $paramsAlteraDadosProrrog = array(
            
            "NRSEQCONTRATO" => $seqContrato,
            "NRSEQPRORRCONT" => $seqprorrog,
            "NRORG"  => $this->Environment->getNrOrg()
        );
        
      $this->PeriodoVigenciaContratoFactory->alteraDadosProrrog($paramsAlteraDadosProrrog);
        
        
    }

}
?>